import { Handler, Context, Callback, APIGatewayEvent } from 'aws-lambda';
import { DynamoDB, config } from 'aws-sdk';
import { Transliterator, ITransliteratorResponse } from './transliterate/transliterate';
import { Calculator } from './calculation/calculation';
import { v1 } from 'uuid';

interface IResponse {
  statusCode: number;
  headers?: object;
  body: string;
}

export const transliterateHandler: Handler = (event: APIGatewayEvent, context: Context | null, callback: Callback) => {
  let transliterator: Transliterator = new Transliterator();
  console.log('Received event:', JSON.stringify(event));
  let inputParams: {[name: string]: string}|null = event.queryStringParameters;
  inputParams = inputParams !== null ? inputParams : {input: ''};
  let input: string = inputParams.input;
  let output: ITransliteratorResponse = transliterator.transliterate(input);

  const response: IResponse = {
    statusCode: 200,
    headers: {'Access-Control-Allow-Origin' : '*'},
    body: JSON.stringify(output)
  };

  callback(undefined, response);
};

export const additionHandler: Handler = (event: any, context: Context, callback: Callback) => {
  try {
    let calculator: Calculator = new Calculator();
    console.log('Received event:', JSON.stringify(event));
    let requestBody = JSON.parse(event.body);
    let a: number = parseInt(requestBody.a, 10);
    let b: number = parseInt(requestBody.b, 10);

    let output: number = calculator.add(a, b);
    const response: IResponse = {
      statusCode: 200,
      headers: {'Access-Control-Allow-Origin' : '*'},
      body: JSON.stringify(output)
    };
    callback(undefined, response);
  } catch (e) {
    handleError(e, callback);
  }
};

///////////////////////////////////////////////////
export const createDynDBRecord: Handler = (event: any, context: Context, callback: Callback) => {
  const dynamoDb = new DynamoDB.DocumentClient();
  const timestamp: number = new Date().getTime();
  const data: any = JSON.parse(event.body);
  if (typeof data.text !== 'string') {
    console.error('Validation Failed');
    callback(undefined, {
      statusCode: 400,
      headers: { 'Content-Type': 'text/plain' },
      body: 'Couldn\'t create the todo item.',
    });
    return;
  }

  const params = {
    TableName: process.env.DYNAMODB_TABLE,
    Item: {
      id: v1(),
      text: data.text,
      checked: false,
      createdAt: timestamp,
      updatedAt: timestamp,
    },
  };

  config.region = 'us-west-2';
  // write the todo to the database
  dynamoDb.put(params, (error) => {
    // handle potential errors
    if (error) {
      console.error(error);
      callback(undefined, {
        statusCode: error.statusCode || 501,
        headers: { 'Content-Type': 'text/plain' },
        body: 'Couldn\'t create the todo item.',
      });
      return;
    }

    // create a response
    const response = {
      statusCode: 200,
      body: JSON.stringify(params.Item),
    };
    callback(undefined, response);
  });
};

let handleError = function(error: Error, callback: Callback) {
  const response: IResponse = {
    statusCode: 500,
    body: JSON.stringify(error)
  };
  callback(error, response);
};